@@ -3,7 +3,7 @@
 #include "image.h"
 
 static const nv_image_t rotation_orientation_vert_flipped_image = {
-  90, 120, 3,
+  90, 120, 3, 0,
   "\377\0\0\0\377\0\0\0\222\0\0\0\1\37\246\37\217)\334)\2&\262Q\35M\356\303"
   "\34E\375\1\25""4\277\204\0\0\0\221.\337.\1+\271\\\304\40N\375\204\0\0\0\222"
   "4\3424\304#W\375\204\0\0\0\2229\3449\1)s\346\303'`\375\204\0\0\0\221>\347"
